@@ -1,6 +1,8 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* libwps
  * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2003-2004 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2004-2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  * Copyright (C) 2006, 2007 Andrew Ziem (andrewziem users sourceforge net)
  *
  * This library is free software; you can redistribute it and/or
@@ -45,23 +47,25 @@ If you are interrested in the structure of libwps itself, this whole document
 would be a good starting point for exploring the interals of libwps. Mind that
 this document is a work-in-progress, and will most likely not cover libwps for
 the full 100%.
-*/
+
+ \warning When compiled with -DDEBUG_WITH__FILES, code is added to store the results of the parsing in different files: one file by Ole parts and some files to store the read pictures. These files are created in the current repository, therefore it is recommended to launch the tests in an empty repository...*/
 
 /**
 Analyzes the content of an input stream to see if it can be parsed
-\param input The input stream
+\param ip The input stream
 \return A confidence value which represents the likelyhood that the content from
 the input stream can be parsed
 */
-WPSConfidence WPSDocument::isFileFormatSupported(WPXInputStream *input)
+WPSConfidence WPSDocument::isFileFormatSupported(WPXInputStream *ip)
 {
 	WPSConfidence confidence = WPS_CONFIDENCE_NONE;
 
 	WPS_DEBUG_MSG(("WPSDocument::isFileFormatSupported()\n"));
-	
+	WPSHeaderPtr header;
+	shared_ptr<WPXInputStream > input(ip, WPS_shared_ptr_noop_deleter<WPXInputStream>());
 	try
 	{
-		WPSHeader *header = WPSHeader::constructHeader(input);
+		header.reset(WPSHeader::constructHeader(input));
 
 		if (!header)
 			return WPS_CONFIDENCE_NONE;
@@ -69,26 +73,25 @@ WPSConfidence WPSDocument::isFileFormatSupported(WPXInputStream *input)
 		switch (header->getMajorVersion())
 		{
 
-			case 8:
-			case 7:
-			case 4:
-				confidence = WPS_CONFIDENCE_EXCELLENT;
-				break;
+		case 8:
+		case 7:
+		case 4:
+			confidence = WPS_CONFIDENCE_EXCELLENT;
+			break;
 
-			case 5:
-			case 2:
-				confidence = WPS_CONFIDENCE_GOOD;
-				break;
+		case 5:
+		case 2:
+			confidence = WPS_CONFIDENCE_GOOD;
+			break;
 		}
-		DELETEP(header);
 
 		return confidence;
 	}
-	catch (FileException)
+	catch (libwps::FileException)
 	{
 		WPS_DEBUG_MSG(("File exception trapped\n"));
 	}
-	catch (ParseException)
+	catch (libwps::ParseException)
 	{
 		WPS_DEBUG_MSG(("Parse exception trapped\n"));
 	}
@@ -105,16 +108,19 @@ WPSConfidence WPSDocument::isFileFormatSupported(WPXInputStream *input)
 Parses the input stream content. It will make callbacks to the functions provided by a
 WPXDocumentInterface class implementation when needed. This is often commonly called the
 'main parsing routine'.
-\param input The input stream
+\param ip The input stream
 \param documentInterface A WPSListener implementation
 */
-WPSResult WPSDocument::parse(WPXInputStream *input, WPXDocumentInterface *documentInterface)
+WPSResult WPSDocument::parse(WPXInputStream *ip, WPXDocumentInterface *documentInterface)
 {
 	WPSResult error = WPS_OK;
 
+	WPSHeaderPtr header;
+	shared_ptr<WPSParser> parser;
+	shared_ptr<WPXInputStream > input(ip, WPS_shared_ptr_noop_deleter<WPXInputStream>());
 	try
 	{
-		WPSHeader *header = WPSHeader::constructHeader(input);
+		header.reset(WPSHeader::constructHeader(input));
 
 		if (!header)
 			return WPS_UNKNOWN_ERROR;
@@ -122,38 +128,37 @@ WPSResult WPSDocument::parse(WPXInputStream *input, WPXDocumentInterface *docume
 		switch (header->getMajorVersion())
 		{
 
-			case 8:
-			case 7:
-			case 6:
-			case 5:
-			{
-				WPS8Parser *parser = new WPS8Parser(header->getInput(), header);
-				parser->parse(documentInterface);
-				DELETEP(parser);		
-				break;
-			}
+		case 8:
+		case 7:
+		case 6:
+		case 5:
+		{
+			parser.reset(new WPS8Parser(header->getInput(), header));
+			if (!parser) return WPS_UNKNOWN_ERROR;
+			parser->parse(documentInterface);
+			break;
+		}
 
-			case 4:
-			case 3:
-			case 2:
-			{
-				WPS4Parser *parser = new WPS4Parser(header->getInput(), header);
-				parser->parse(documentInterface);
-				DELETEP(parser);	
-				break;
-			}
+		case 4:
+		case 3:
+		case 2:
+		{
+			parser.reset(new WPS4Parser(header->getInput(), header));
+			if (!parser) return WPS_UNKNOWN_ERROR;
+			parser->parse(documentInterface);
+			break;
+		}
 		}
-		delete header;
 	}
-	catch (FileException)
+	catch (libwps::FileException)
 	{
 		WPS_DEBUG_MSG(("File exception trapped\n"));
 		error = WPS_FILE_ACCESS_ERROR;
 	}
-	catch (ParseException)
+	catch (libwps::ParseException)
 	{
 		WPS_DEBUG_MSG(("Parse exception trapped\n"));
-                error = WPS_PARSE_ERROR;
+		error = WPS_PARSE_ERROR;
 	}
 	catch (...)
 	{
@@ -164,3 +169,4 @@ WPSResult WPSDocument::parse(WPXInputStream *input, WPXDocumentInterface *docume
 
 	return error;
 }
+/* vim:set shiftwidth=4 softtabstop=4 noexpandtab: */
